﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  /// <summary>
  /// Represents a weight calculator that gives every classification solution the same weight.
  /// </summary>
  [StorableClass]
  [Item("MajorityVoteWeightCalculator", "Represents a weight calculator that gives every classification solution the same weight.")]
  public class MajorityVoteWeightCalculator : ClassificationWeightCalculator {

    public MajorityVoteWeightCalculator()
      : base() {
    }

    [StorableConstructor]
    protected MajorityVoteWeightCalculator(bool deserializing) : base(deserializing) { }
    protected MajorityVoteWeightCalculator(MajorityVoteWeightCalculator original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MajorityVoteWeightCalculator(this, cloner);
    }

    protected override IEnumerable<double> CalculateWeights(IEnumerable<IClassificationSolution> classificationSolutions) {
      return Enumerable.Repeat<double>(1, classificationSolutions.Count());
    }

    public override double GetConfidence(IEnumerable<IClassificationSolution> solutions, int index, double estimatedClassValue, CheckPoint handler) {
      if (solutions.Count() < 1)
        return double.NaN;
      var votingSolutions = solutions.Where(s => handler(s.ProblemData, index));
      if (votingSolutions.Count() < 1)
        return double.NaN;
      Dataset dataset = votingSolutions.First().ProblemData.Dataset;
      int correctEstimated = votingSolutions.Select(s => s.Model.GetEstimatedClassValues(dataset, Enumerable.Repeat(index, 1)).First())
                                      .Where(x => x.Equals(estimatedClassValue))
                                      .Count();
      return ((double)correctEstimated / (double)votingSolutions.Count() - 0.5) * 2;
    }

    public override IEnumerable<double> GetConfidence(IEnumerable<IClassificationSolution> solutions, IEnumerable<int> indices, IEnumerable<double> estimatedClassValue, CheckPoint handler) {
      if (solutions.Count() < 1)
        return Enumerable.Repeat(double.NaN, indices.Count());

      List<int> indicesList = indices.ToList();
      Dataset dataset = solutions.First().ProblemData.Dataset;
      var solValues = solutions.ToDictionary(x => x, x => x.Model.GetEstimatedClassValues(dataset, indicesList).ToArray());
      double[] estimatedClassValueArr = estimatedClassValue.ToArray();
      double correctEstimated;
      double[] confidences = new double[indices.Count()];

      for (int i = 0; i < indicesList.Count; i++) {
        var votingSolutions = solValues.Where(x => handler(x.Key.ProblemData, indicesList[i]));
        correctEstimated = votingSolutions.Where(x => DoubleExtensions.IsAlmost(x.Value[i], estimatedClassValueArr[i])).Count();
        confidences[i] = (correctEstimated / (double)votingSolutions.Count() - 0.5) * 2;
      }

      return confidences;
    }
  }
}
